@@ -1,7 +1,9 @@
 require 'puppet/application'
-require 'puppet/run'
 require 'puppet/daemon'
 require 'puppet/util/pidlock'
+require 'puppet/agent'
+require 'puppet/configurer'
+require 'puppet/ssl/oids'
 
 class Puppet::Application::Agent < Puppet::Application
 
@@ -19,7 +21,7 @@ class Puppet::Application::Agent < Puppet::Application
   def preinit
     # Do an initial trap, so that cancels don't get a stack trace.
     Signal.trap(:INT) do
-      $stderr.puts "Cancelling startup"
+      $stderr.puts _("Cancelling startup")
       exit(0)
     end
 
@@ -31,12 +33,12 @@ class Puppet::Application::Agent < Puppet::Application
       :setdest => false,
       :enable => false,
       :disable => false,
-      :client => true,
       :fqdn => nil,
       :serve => [],
       :digest => 'SHA256',
       :graph => true,
       :fingerprint => false,
+      :sourceaddress => nil,
     }.each do |opt,val|
       options[opt] = val
     end
@@ -58,9 +60,7 @@ class Puppet::Application::Agent < Puppet::Application
   option("--fingerprint")
   option("--digest DIGEST")
 
-  option("--no-client") do |arg|
-    options[:client] = false
-  end
+  option("--sourceaddress IP_ADDRESS")
 
   option("--detailed-exitcodes") do |arg|
     options[:detailed_exitcodes] = true
@@ -74,10 +74,18 @@ class Puppet::Application::Agent < Puppet::Application
     options[:waitforcert] = arg.to_i
   end
 
-  def help
-    <<-'HELP'
+  option("--job-id ID") do |arg|
+    options[:job_id] = arg
+  end
 
-puppet-agent(8) -- The puppet agent daemon
+  def summary
+    _("The puppet agent daemon")
+  end
+
+  def help
+    <<-HELP
+
+puppet-agent(8) -- #{summary}
 ========
 
 SYNOPSIS
@@ -91,11 +99,11 @@ similar), or run interactively for testing purposes.
 
 USAGE
 -----
-puppet agent [--certname <name>] [-D|--daemonize|--no-daemonize]
-  [-d|--debug] [--detailed-exitcodes] [--digest <digest>] [--disable [message]] [--enable]
-  [--fingerprint] [-h|--help] [-l|--logdest syslog|<file>|console]
-  [--no-client] [--noop] [-o|--onetime] [-t|--test]
-  [-v|--verbose] [-V|--version] [-w|--waitforcert <seconds>]
+puppet agent [--certname <NAME>] [-D|--daemonize|--no-daemonize]
+  [-d|--debug] [--detailed-exitcodes] [--digest <DIGEST>] [--disable [MESSAGE]] [--enable]
+  [--fingerprint] [-h|--help] [-l|--logdest syslog|eventlog|<ABS FILEPATH>|console]
+  [--masterport <PORT>] [--noop] [-o|--onetime] [--sourceaddress <IP_ADDRESS>] [-t|--test]
+  [-v|--verbose] [-V|--version] [-w|--waitforcert <SECONDS>]
 
 
 DESCRIPTION
@@ -121,14 +129,14 @@ go into the background, attempt to get a signed certificate, and retrieve
 and apply its configuration every 30 minutes.
 
 Some flags are meant specifically for interactive use -- in particular,
-'test', 'tags' or 'fingerprint' are useful. 'test' enables verbose
-logging, causes the daemon to stay in the foreground, exits if the
-server's configuration is invalid (this happens if, for instance, you've
-left a syntax error on the server), and exits after running the
-configuration once (rather than hanging around as a long-running
-process).
+'test', 'tags' and 'fingerprint' are useful.
 
-'tags' allows you to specify what portions of a configuration you want
+'--test' does a single run in the foreground with verbose logging, then exits.
+It will also exit if it can't get a valid catalog. The exit code after running
+with '--test' is 0 if the catalog was successfully applied, and 1 if the run
+either failed or wasn't attempted (due to another run already in progress).
+
+'--tags' allows you to specify what portions of a configuration you want
 to apply. Puppet elements are tagged with all of the class or definition
 names that contain them, and you can use the 'tags' flag to specify one
 of these names, causing only configuration elements contained within
@@ -139,7 +147,7 @@ class, and call puppet with '--tags ntpd', which would only apply that
 small portion of the configuration during your testing, rather than
 applying the whole thing.
 
-'fingerprint' is a one-time flag. In this mode 'puppet agent' will run
+'--fingerprint' is a one-time flag. In this mode 'puppet agent' will run
 once and display on the console (and in the log) the current certificate
 (or certificate request) fingerprint. Providing the '--digest' option
 allows to use a different digest algorithm to generate the fingerprint.
@@ -158,7 +166,7 @@ specify '--server <servername>' as an argument. Boolean settings translate into
 '--setting' and '--no-setting' pairs.
 
 See the configuration file documentation at
-http://docs.puppetlabs.com/references/stable/configuration.html for the
+https://puppet.com/docs/puppet/latest/configuration.html for the
 full list of acceptable settings. A commented list of all settings can also be
 generated by running puppet agent with '--genconfig'.
 
@@ -184,10 +192,20 @@ generated by running puppet agent with '--genconfig'.
   Enable full debugging.
 
 * --detailed-exitcodes:
-  Provide transaction information via exit codes. If this is enabled, an exit
-  code of '2' means there were changes, an exit code of '4' means there were
-  failures during the transaction, and an exit code of '6' means there were both
-  changes and failures.
+  Provide extra information about the run via exit codes; only works if '--test'
+  or '--onetime' is also specified. If enabled, 'puppet agent' will use the
+  following exit codes:
+
+  0: The run succeeded with no changes or failures; the system was already in
+  the desired state.
+
+  1: The run failed, or wasn't attempted due to another run already in progress.
+
+  2: The run succeeded, and some resources were changed.
+
+  4: The run succeeded, and some resources failed.
+
+  6: The run succeeded, and included both changes and failures.
 
 * --digest:
   Change the certificate fingerprinting digest algorithm. The default is
@@ -225,21 +243,25 @@ generated by running puppet agent with '--genconfig'.
 * --help:
   Print this help message
 
+* --job-id:
+  Attach the specified job id to the catalog request and the report used for
+  this agent run. This option only works when '--onetime' is used.
+
 * --logdest:
-  Where to send messages. Choose between syslog, the console, and a log
-  file. Defaults to sending messages to syslog, or the console if
-  debugging or verbosity is enabled.
+  Where to send log messages. Choose between 'syslog' (the POSIX syslog
+  service), 'eventlog' (the Windows Event Log), 'console', or the path to a log
+  file. If debugging or verbosity is enabled, this defaults to 'console'.
+  Otherwise, it defaults to 'syslog' on POSIX systems and 'eventlog' on Windows.
+
+  A path ending with '.json' will receive structured output in JSON format. The
+  log file will not have an ending ']' automatically written to it due to the
+  appending nature of logging. It must be appended manually to make the content
+  valid JSON.
 
 * --masterport:
   The port on which to contact the puppet master.
   (This is a Puppet setting, and can go in puppet.conf.)
 
-* --no-client:
-  Do not create a config client. This will cause the daemon to start
-  but not check configuration unless it is triggered with `puppet
-  kick`. This only makes sense when puppet agent is being run with
-  listen = true in puppet.conf or was started with the `--listen` option.
-
 * --noop:
   Use 'noop' mode where the daemon runs in a no-op or dry-run mode. This
   is useful for seeing what changes Puppet will make without actually
@@ -254,10 +276,14 @@ generated by running puppet agent with '--genconfig'.
   (This is a Puppet setting, and can go in puppet.conf. Note the special 'no-'
   prefix for boolean settings on the command line.)
 
+* --sourceaddress:
+  Set the source IP address for transactions. This defaults to automatically selected.
+  (This is a Puppet setting, and can go in puppet.conf.)
+
 * --test:
   Enable the most common options used for testing. These are 'onetime',
-  'verbose', 'ignorecache', 'no-daemonize', 'no-usecacheonfailure',
-  'detailed-exitcodes', 'no-splay', and 'show_diff'.
+  'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
+  'no-splay', and 'show_diff'.
 
 * --verbose:
   Turn on verbose reporting.
@@ -302,7 +328,7 @@ Luke Kanies
 
 COPYRIGHT
 ---------
-Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
+Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
 
     HELP
   end
@@ -315,6 +341,11 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
       # waiting for certificates so that we don't block
       daemon = daemonize_process_when(Puppet[:daemonize])
 
+      # Setup signal traps immediately after daemonization so we clean up the daemon
+      daemon.set_signal_traps
+
+      log_config if Puppet[:daemonize]
+
       wait_for_certificates
 
       if Puppet[:onetime]
@@ -325,34 +356,32 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
     end
   end
 
+  def log_config
+    #skip also config reading and parsing if debug is not enabled
+    return unless Puppet::Util::Log.sendlevel?(:debug)
+
+    Puppet.settings.stringify_settings(:agent, :all).each_pair do |k,v|
+      next if k.include?("password") || v.to_s.empty?
+      Puppet.debug("Using setting: #{k}=#{v}")
+    end
+  end
+
   def fingerprint
     host = Puppet::SSL::Host.new
     unless cert = host.certificate || host.certificate_request
-      $stderr.puts "Fingerprint asked but no certificate nor certificate request have yet been issued"
+      $stderr.puts _("Fingerprint asked but no certificate nor certificate request have yet been issued")
       exit(1)
       return
     end
     unless digest = cert.digest(options[:digest].to_s)
-      raise ArgumentError, "Could not get fingerprint for digest '#{options[:digest]}'"
+      raise ArgumentError, _("Could not get fingerprint for digest '%{digest}'") % { digest: options[:digest] }
     end
     puts digest.to_s
   end
 
   def onetime(daemon)
-    if Puppet[:listen]
-      Puppet.notice "Ignoring --listen on onetime run"
-    end
-
-    unless options[:client]
-      Puppet.err "onetime is specified but there is no client"
-      exit(43)
-      return
-    end
-
-    daemon.set_signal_traps
-
     begin
-      exitstatus = daemon.agent.run
+      exitstatus = daemon.agent.run(:job_id => options[:job_id])
     rescue => detail
       Puppet.log_exception(detail)
     end
@@ -369,17 +398,12 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
   end
 
   def main(daemon)
-    if Puppet[:listen]
-      setup_listen(daemon)
-    end
-    Puppet.notice "Starting Puppet client version #{Puppet.version}"
-
+    Puppet.notice _("Starting Puppet client version %{version}") % { version: Puppet.version }
     daemon.start
   end
 
   # Enable all of the most common test options.
   def setup_test
-    Puppet.settings.handlearg("--ignorecache")
     Puppet.settings.handlearg("--no-usecacheonfailure")
     Puppet.settings.handlearg("--no-splay")
     Puppet.settings.handlearg("--show_diff")
@@ -390,22 +414,22 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
   end
 
   def setup
+    raise ArgumentError, _("The puppet agent command does not take parameters") unless command_line.args.empty?
+
     setup_test if options[:test]
 
     setup_logs
 
     exit(Puppet.settings.print_configs ? 0 : 1) if Puppet.settings.print_configs?
 
+    Puppet::SSL::Oids.register_puppet_oids
+
     if options[:fqdn]
       Puppet[:certname] = options[:fqdn]
     end
 
     Puppet.settings.use :main, :agent, :ssl
 
-    # Always ignoreimport for agent. It really shouldn't even try to import,
-    # but this is just a temporary band-aid.
-    Puppet[:ignoreimport] = true
-
     Puppet::Transaction::Report.indirection.terminus_class = :rest
     # we want the last report to be persisted locally
     Puppet::Transaction::Report.indirection.cache_class = :yaml
@@ -435,30 +459,12 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
     exit(0)
   end
 
-  def setup_listen(daemon)
-    Puppet.warning "Puppet --listen / kick is deprecated. See http://links.puppetlabs.com/puppet-kick-deprecation"
-    unless Puppet::FileSystem::File.exist?(Puppet[:rest_authconfig])
-      Puppet.err "Will not start without authorization file #{Puppet[:rest_authconfig]}"
-      exit(14)
-    end
-
-    require 'puppet/network/server'
-    # No REST handlers yet.
-    server = Puppet::Network::Server.new(Puppet[:bindaddress], Puppet[:puppetport])
-
-    daemon.server = server
-  end
-
   def setup_agent
-    # We need to make the client either way, we just don't start it
-    # if --no-client is set.
-    require 'puppet/agent'
-    require 'puppet/configurer'
     agent = Puppet::Agent.new(Puppet::Configurer, (not(Puppet[:onetime])))
 
     enable_disable_client(agent) if options[:enable] or options[:disable]
 
-    @agent = agent if options[:client]
+    @agent = agent
   end
 
   def daemonize_process_when(should_daemonize)
