@@ -25,7 +25,7 @@ class EvaluatingParser
     # Also a possible improvement (if the YAML parser returns positions) is to provide correct output of position.
     #
     begin
-      assert_and_report(parser.parse_string(s, file_source))
+      assert_and_report(parser.parse_string(s, file_source)).model
     rescue Puppet::ParseErrorWithIssue => e
       raise e
     rescue Puppet::ParseError => e
@@ -38,7 +38,7 @@ class EvaluatingParser
   def parse_file(file)
     @file_source = file
     clear()
-    assert_and_report(parser.parse_file(file))
+    assert_and_report(parser.parse_file(file)).model
   end
 
   def evaluate_string(scope, s, file_source = nil)
@@ -55,7 +55,7 @@ class EvaluatingParser
 
   # Create a closure that can be called in the given scope
   def closure(model, scope)
-    Evaluator::Closure.new(evaluator, model, scope)
+    Evaluator::Closure::Dynamic.new(evaluator, model, scope)
   end
 
   def evaluate(scope, model)
@@ -80,7 +80,7 @@ class EvaluatingParser
   end
 
   def convert_to_3x(object, scope)
-    val = evaluator.convert(object, scope, nil)
+    evaluator.convert(object, scope, nil)
   end
 
   def validate(parse_result)
@@ -99,10 +99,10 @@ class EvaluatingParser
 
   def assert_and_report(parse_result)
     return nil unless parse_result
-    if parse_result.source_ref.nil? || parse_result.source_ref == ''
-      parse_result.source_ref = @file_source
+    if parse_result['source_ref'].nil? || parse_result['source_ref'] == ''
+      parse_result['source_ref'] = @file_source
     end
-    validation_result = validate(parse_result)
+    validation_result = validate(parse_result.model)
 
     IssueReporter.assert_and_report(validation_result,
                                           :emit_warnings => true)
