@@ -84,7 +84,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
       sc = StringScanner.new(line)
       cmd = sc.scan(/\w+|==|!=/)
       formals = COMMANDS[cmd]
-      fail("Unknown command #{cmd}") unless formals
+      fail(_("Unknown command %{cmd}") % { cmd: cmd }) unless formals
       argline << cmd
       narg = 0
       formals.each do |f|
@@ -102,12 +102,12 @@ Puppet::Type.type(:augeas).provide(:augeas) do
             nbracket -= 1 if ch == "]"
             inSingleTick = !inSingleTick if ch == "'"
             inDoubleTick = !inDoubleTick if ch == "\""
-            fail("unmatched [") if nbracket < 0
-          end until ((nbracket == 0 && !inSingleTick && !inDoubleTick && (ch =~ /\s/)) || sc.eos?)
+            fail(_("unmatched [")) if nbracket < 0
+          end until ((nbracket == 0 && !inSingleTick && !inDoubleTick && (ch =~ /\s/)) || sc.eos?) # rubocop:disable Lint/Loop
             len = sc.pos - start
             len -= 1 unless sc.eos?
           unless p = sc.string[start, len]
-            fail("missing path argument #{narg} for #{cmd}")
+            fail(_("missing path argument %{narg} for %{cmd}") % { narg: narg, cmd: cmd })
           end
           # Rip off any ticks if they are there.
           p = p[1, (p.size - 2)] if p[0,1] == "'" || p[0,1] == "\""
@@ -122,16 +122,23 @@ Puppet::Type.type(:augeas).provide(:augeas) do
           if delim == "'" || delim == "\""
             sc.getch
             argline << sc.scan(/([^\\#{delim}]|(\\.))*/)
+            # Unescape the delimiter so it's actually possible to have a
+            # literal delim inside the string. We only unescape the
+            # delimeter and not every backslash-escaped character so that
+            # things like escaped spaces '\ ' get passed through because
+            # Augeas needs to see them. If we unescaped them, too, users
+            # would be forced to double-escape them
+            argline.last.gsub!(/\\(#{delim})/, '\1')
             sc.getch
           else
             argline << sc.scan(/[^\s]+/)
           end
-          fail("missing string argument #{narg} for #{cmd}") unless argline[-1]
+          fail(_("missing string argument %{narg} for %{cmd}") % { narg: narg, cmd: cmd }) unless argline[-1]
         elsif f == :comparator
           argline << sc.scan(/(==|!=|=~|<=|>=|<|>)/)
           unless argline[-1]
             puts sc.rest
-            fail("invalid comparator for command #{cmd}")
+            fail(_("invalid comparator for command %{cmd}") % { cmd: cmd })
           end
         elsif f == :int
           argline << sc.scan(/\d+/).to_i
@@ -168,7 +175,6 @@ Puppet::Type.type(:augeas).provide(:augeas) do
       glob_avail = !aug.match("/augeas/version/pathx/functions/glob").empty?
       opt_ctx = resource[:context].match("^/files/[^'\"\\[\\]]+$") if resource[:context]
 
-      restricted = false
       if resource[:incl]
         aug.set("/augeas/load/Xfm/lens", resource[:lens])
         aug.set("/augeas/load/Xfm/incl", resource[:incl])
@@ -203,7 +209,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
 
   def is_numeric?(s)
     case s
-    when Fixnum
+    when Integer
       true
     when String
       s.match(/\A[+-]?\d+?(\.\d+)?\Z/n) == nil ? false : true
@@ -219,8 +225,8 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     return_value = false
 
     #validate and tear apart the command
-    fail ("Invalid command: #{cmd_array.join(" ")}") if cmd_array.length < 4
-    cmd = cmd_array.shift
+    fail (_("Invalid command: %{cmd}") % { cmd: cmd_array.join(" ") }) if cmd_array.length < 4
+    _ = cmd_array.shift
     path = cmd_array.shift
     comparator = cmd_array.shift
     arg = cmd_array.join(" ")
@@ -250,8 +256,8 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     return_value = false
 
     #validate and tear apart the command
-    fail("Invalid command: #{cmd_array.join(" ")}") if cmd_array.length < 3
-    cmd = cmd_array.shift
+    fail(_("Invalid command: %{cmd}") % { cmd: cmd_array.join(" ") }) if cmd_array.length < 3
+    _ = cmd_array.shift
     path = cmd_array.shift
 
     # Need to break apart the clause
@@ -260,7 +266,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
 
     #Get the match paths from augeas
     result = @aug.match(path) || []
-    fail("Error trying to get path '#{path}'") if (result == -1)
+    fail(_("Error trying to get path '%{path}'") % { path: path }) if (result == -1)
 
     #Get the values of the match paths from augeas
     values = result.collect{|r| @aug.get(r)}
@@ -275,18 +281,18 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     when "=="
       begin
         arg = clause_array.shift
-        new_array = eval arg
+        new_array = to_array(arg)
         return_value = (values == new_array)
       rescue
-        fail("Invalid array in command: #{cmd_array.join(" ")}")
+        fail(_("Invalid array in command: %{cmd}") % { cmd: cmd_array.join(" ") })
       end
     when "!="
       begin
         arg = clause_array.shift
-        new_array = eval arg
+        new_array = to_array(arg)
         return_value = (values != new_array)
       rescue
-        fail("Invalid array in command: #{cmd_array.join(" ")}")
+        fail(_("Invalid array in command: %{cmd}") % { cmd: cmd_array.join(" ") })
       end
     end
     !!return_value
@@ -298,8 +304,8 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     return_value = false
 
     #validate and tear apart the command
-    fail("Invalid command: #{cmd_array.join(" ")}") if cmd_array.length < 3
-    cmd = cmd_array.shift
+    fail(_("Invalid command: %{cmd}") % { cmd: cmd_array.join(" ") }) if cmd_array.length < 3
+    _ = cmd_array.shift
     path = cmd_array.shift
 
     # Need to break apart the clause
@@ -308,12 +314,12 @@ Puppet::Type.type(:augeas).provide(:augeas) do
 
     #Get the values from augeas
     result = @aug.match(path) || []
-    fail("Error trying to match path '#{path}'") if (result == -1)
+    fail(_("Error trying to match path '%{path}'") % { path: path }) if (result == -1)
 
     # Now do the work
     case verb
     when "size"
-      fail("Invalid command: #{cmd_array.join(" ")}") if clause_array.length != 2
+      fail(_("Invalid command: %{cmd}") % { cmd: cmd_array.join(" ") }) if clause_array.length != 2
       comparator = clause_array.shift
       arg = clause_array.shift
       case comparator
@@ -331,18 +337,18 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     when "=="
       begin
         arg = clause_array.shift
-        new_array = eval arg
+        new_array = to_array(arg)
         return_value = (result == new_array)
       rescue
-        fail("Invalid array in command: #{cmd_array.join(" ")}")
+        fail(_("Invalid array in command: %{cmd}") % { cmd: cmd_array.join(" ") })
       end
     when "!="
       begin
         arg = clause_array.shift
-        new_array = eval arg
+        new_array = to_array(arg)
         return_value = (result != new_array)
       rescue
-        fail("Invalid array in command: #{cmd_array.join(" ")}")
+        fail(_("Invalid array in command: %{cmd}") % { cmd: cmd_array.join(" ") })
       end
     end
     !!return_value
@@ -378,7 +384,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     errors = @aug.match("/augeas//error")
     unless errors.empty?
       if path && !@aug.match(path).empty?
-        warning("Loading failed for one or more files, see debug for /augeas//error output")
+        warning(_("Loading failed for one or more files, see debug for /augeas//error output"))
       else
         debug("Loading failed for one or more files, output from /augeas//error:")
       end
@@ -420,7 +426,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
           when "match"; return_value = process_match(cmd_array)
           end
         rescue StandardError => e
-          fail("Error sending command '#{command}' with params #{cmd_array[1..-1].inspect}/#{e.message}")
+          fail(_("Error sending command '%{command}' with params %{param}/%{message}") % { command: command, param: cmd_array[1..-1].inspect, message: e.message })
         end
       end
 
@@ -435,7 +441,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
           save_result = @aug.save
           unless save_result
             print_put_errors
-            fail("Saving failed, see debug")
+            fail(_("Saving failed, see debug"))
           end
 
           saved_files = @aug.match("/augeas/events/saved")
@@ -477,7 +483,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
     do_execute_changes
     unless @aug.save
       print_put_errors
-      fail("Save failed, see debug")
+      fail(_("Save failed, see debug"))
     end
 
     :executed
@@ -489,7 +495,7 @@ Puppet::Type.type(:augeas).provide(:augeas) do
   def do_execute_changes
     commands = parse_commands(resource[:changes])
     commands.each do |cmd_array|
-      fail("invalid command #{cmd_array.join[" "]}") if cmd_array.length < 2
+      fail(_("invalid command %{cmd}") % { value0: cmd_array.join[" "] }) if cmd_array.length < 2
       command = cmd_array[0]
       cmd_array.shift
       begin
@@ -497,37 +503,37 @@ Puppet::Type.type(:augeas).provide(:augeas) do
           when "set"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.set(cmd_array[0], cmd_array[1])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
           when "setm"
             if aug.respond_to?(command)
               debug("sending command '#{command}' with params #{cmd_array.inspect}")
               rv = aug.setm(cmd_array[0], cmd_array[1], cmd_array[2])
-              fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (rv == -1)
+              fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (rv == -1)
             else
-              fail("command '#{command}' not supported in installed version of ruby-augeas")
+              fail(_("command '%{command}' not supported in installed version of ruby-augeas") % { command: command })
             end
           when "rm", "remove"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.rm(cmd_array[0])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (rv == -1)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (rv == -1)
           when "clear"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.clear(cmd_array[0])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
           when "clearm"
             # Check command exists ... doesn't currently in ruby-augeas 0.4.1
             if aug.respond_to?(command)
               debug("sending command '#{command}' with params #{cmd_array.inspect}")
               rv = aug.clearm(cmd_array[0], cmd_array[1])
-              fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+              fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
             else
-              fail("command '#{command}' not supported in installed version of ruby-augeas")
+              fail(_("command '%{command}' not supported in installed version of ruby-augeas") % { command: command })
             end
           when "touch"
             debug("sending command '#{command}' (match, set) with params #{cmd_array.inspect}")
             if aug.match(cmd_array[0]).empty?
               rv = aug.clear(cmd_array[0])
-              fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+              fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
             end
           when "insert", "ins"
             label = cmd_array[0]
@@ -536,32 +542,226 @@ Puppet::Type.type(:augeas).provide(:augeas) do
             case where
               when "before"; before = true
               when "after"; before = false
-              else fail("Invalid value '#{where}' for where param")
+              else fail(_("Invalid value '%{where}' for where param") % { where: where })
             end
             debug("sending command '#{command}' with params #{[label, where, path].inspect}")
             rv = aug.insert(path, label, before)
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (rv == -1)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (rv == -1)
           when "defvar"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.defvar(cmd_array[0], cmd_array[1])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
           when "defnode"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.defnode(cmd_array[0], cmd_array[1], cmd_array[2])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (!rv)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (!rv)
           when "mv", "move"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.mv(cmd_array[0], cmd_array[1])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (rv == -1)
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (rv == -1)
           when "rename"
             debug("sending command '#{command}' with params #{cmd_array.inspect}")
             rv = aug.rename(cmd_array[0], cmd_array[1])
-            fail("Error sending command '#{command}' with params #{cmd_array.inspect}") if (rv == -1)
-          else fail("Command '#{command}' is not supported")
+            fail(_("Error sending command '%{command}' with params %{params}") % { command: command, params: cmd_array.inspect }) if (rv == -1)
+          else fail(_("Command '%{command}' is not supported") % { command: command })
         end
       rescue StandardError => e
-        fail("Error sending command '#{command}' with params #{cmd_array.inspect}/#{e.message}")
+        fail(_("Error sending command '%{command}' with params %{params}/%{message}") % { command: command, params: cmd_array.inspect, message: e.message })
       end
     end
   end
+
+  def to_array(string)
+    s = StringScanner.new(string)
+    match = array_open(s)
+    raise "Unable to parse array. Unexpected character at: #{s.rest}" if match.nil?
+
+    array_content = array_values(s)
+
+    match = array_close(s)
+    raise "Unable to parse array. Unexpected character at: #{s.rest}" if match.nil?
+
+    array_content
+  end
+  private :to_array
+
+  def array_open(scanner)
+    scanner.scan(/\s*\[\s*/)
+  end
+  private :array_open
+
+  def array_close(scanner)
+    scanner.scan(/\s*\]\s*/)
+  end
+  private :array_close
+
+  def array_separator(scanner)
+    scanner.scan(/\s*,\s*/)
+  end
+  private :array_separator
+
+  def single_quote_unescaped_char(scanner)
+    scanner.scan(/[^'\\]/)
+  end
+  private :single_quote_unescaped_char
+
+  def single_quote_escaped_char(scanner)
+    scanner.scan(/\\(['\\])/) && scanner[1]
+  end
+  private :single_quote_escaped_char
+
+  def single_quote_char(scanner)
+    single_quote_escaped_char(scanner) || single_quote_unescaped_char(scanner)
+  end
+  private :single_quote_char
+
+  def double_quote_unescaped_char(scanner)
+    scanner.scan(/[^"\\]/)
+  end
+  private :double_quote_unescaped_char
+
+  # This handles the possible Ruby escape sequences in double-quoted strings,
+  # except for \M-x, \M-\C-x, \M-\cx, \c\M-x, \c?, and \C-?. The full list of
+  # escape sequences, and their meanings is taken from:
+  # https://github.com/ruby/ruby/blob/90fdfec11a4a42653722e2ce2a672d6e87a57b8e/doc/syntax/literals.rdoc#strings
+  def double_quote_escaped_char(scanner)
+    match = scanner.scan(/\\(["\\abtnvfres0-7xu])/)
+    return nil if match.nil?
+
+    case scanner[1]
+    when '\\' then return '\\'
+    when '"'  then return '"'
+    when 'a'  then return "\a"
+    when 'b'  then return "\b"
+    when 't'  then return "\t"
+    when 'n'  then return "\n"
+    when 'v'  then return "\v"
+    when 'f'  then return "\f"
+    when 'r'  then return "\r"
+    when 'e'  then return "\e"
+    when 's'  then return "\s"
+    when /[0-7]/
+      octal_character = scanner[1]
+      other_digits = scanner.scan(/[0-7]{1,2}/)
+      octal_character << other_digits unless other_digits.nil?
+
+      return octal_character.to_i(8).chr
+    when 'x'
+      hex_character = scanner.scan(/[0-9a-fA-F]{1,2}/)
+      if hex_character.nil?
+        return nil
+      else
+        return hex_character.to_i(16).chr
+      end
+    when 'u'
+      return unicode_short_hex_character(scanner) || unicode_long_hex_characters(scanner)
+    else
+      # Not a valid escape sequence as far as we're concerned.
+      return nil
+    end
+  end
+  private :double_quote_escaped_char
+
+  def unicode_short_hex_character(scanner)
+    unicode_character = scanner.scan(/[0-9a-fA-F]{4}/)
+    if unicode_character.nil?
+      return nil
+    else
+      return [unicode_character.hex].pack 'U'
+    end
+  end
+  private :unicode_short_hex_character
+
+  def unicode_long_hex_characters(scanner)
+    unicode_string = ''
+    return nil unless scanner.scan(/{/)
+
+    loop do
+      char = scanner.scan(/[0-9a-fA-F]{1,6}/)
+      break if char.nil?
+      unicode_string << [char.hex].pack('U')
+
+      separator = scanner.scan(/\s/)
+      break if separator.nil?
+    end
+
+    return nil unless scanner.scan(/}/)
+
+    if unicode_string.empty?
+      return nil
+    else
+      return unicode_string
+    end
+  end
+  private :unicode_long_hex_characters
+
+  def single_quoted_string(scanner)
+    quoted_string = ''
+
+    match = scanner.scan(/'/)
+    return nil if match.nil?
+
+    loop do
+      match = single_quote_char(scanner)
+      break if match.nil?
+
+      quoted_string << match
+    end
+
+    match = scanner.scan(/'/)
+    if match
+      return quoted_string
+    else
+      return nil
+    end
+  end
+  private :single_quoted_string
+
+  def double_quote_char(scanner)
+    double_quote_escaped_char(scanner) || double_quote_unescaped_char(scanner)
+  end
+  private :double_quote_char
+
+  def double_quoted_string(scanner)
+    quoted_string = ''
+
+    match = scanner.scan(/"/)
+    return nil if match.nil?
+
+    loop do
+      match = double_quote_char(scanner)
+      break if match.nil?
+
+      quoted_string << match
+    end
+
+    match = scanner.scan(/"/)
+    if match
+      return quoted_string
+    else
+      return nil
+    end
+  end
+  private :double_quoted_string
+
+  def quoted_string(scanner)
+    single_quoted_string(scanner) || double_quoted_string(scanner)
+  end
+  private :quoted_string
+
+  def array_values(scanner)
+    values = []
+
+    loop do
+      match = quoted_string(scanner)
+      break if match.nil?
+      values << match
+
+      match = array_separator(scanner)
+      break if match.nil?
+    end
+
+    values
+  end
+  private :array_values
 end
